;; autocompletion
(eval-after-load "notmuch-address"
  '(progn
     (notmuch-address-message-insinuate)))

; don't remember what that is
(add-hook 'notmuch-show-hook 'visual-line-mode)

(defun notmuch-load-bug (bug)
  "download and show a Debian bug report with notmuch-slurp-debbug"
  (interactive "MDebian bug number: ")
  (message "downloading debian bug %s..." bug)
  (call-process "notmuch-slurp-debbug" nil nil nil bug)
  (message "running notmuch-poll...")
  (notmuch-poll)
  (message "searching for bug %s..." bug)
  (notmuch-search bug)
  (message "bug %s loaded, but really you should use upstream notmuch-slurp-debbug" bug))

;; attachment checks
;;
;; should be sent upstream, but needs unit tests in test/T310-emacs.sh
(defcustom notmuch-message-attach-regex
  "\\b\\(attache\?ment\\|attached\\|attach\\|pi[èe]ce\s+jointe?\\)\\b"
  "Pattern of text announcing there should be an attachment.

This is used by `notmuch-message-check-attach' to check email
bodies for words that might indicate the email should have an
attachement. If the pattern matches and there is no attachment (a
`<#part ...>' magic block), notmuch will show a confirmation
prompt before sending the email.

The default regular expression is deliberately liberal: we prefer
false positive than forgotten attachments. This should be
customized for non-english languages and notmuch welcomes
additions to the pattern for your native language, unless it
conflicts with common words in other languages."
  :type '(regexp)
  :group 'notmuch-send)

(defun notmuch-message-check-attach ()
  """Check for missing attachments.

This is normally added to `message-send-hook' and is configured
through `notmuch-message-attach-regex'."""
  (save-excursion ;; XXX: this fails somehow: point is at the end of the buffer on error
    (goto-char (point-min))
    (if (re-search-forward notmuch-message-attach-regex nil t)
        (progn
          (goto-char (point-min))
          (unless (re-search-forward "<#part [^>]*filename=[^>]*>" nil t)
            (or (y-or-n-p "Email seem to refer to attachment, but nothing attached, send anyways?")
                (error "No attachment found, aborting")))))))

(defcustom notmuch-mua-attachment-regexp
  "\\b\\(attache\?ment\\|attached\\|attach\\|pi[èe]ce\s+jointe?\\)\\b"
  "Message body text indicating that an attachment is expected.

This is not used unless `notmuch-mua-attachment-check' is added
to `notmuch-mua-send-hook'.")

(defun notmuch-mua-attachment-check ()
  "Signal an error if the message text indicates that an
attachment is expected but no MML referencing an attachment is
found.

Typically this is added to `notmuch-mua-send-hook'."
  (when (and
	 ;; When the message mentions attachment...
	 (save-excursion
	   (message-goto-body)
	   (loop while (re-search-forward notmuch-mua-attachment-regexp (point-max) t)
		 ;; For every instance of the "attachment" string
		 ;; found, examine the text properties. If the text
		 ;; has either a `face' or `syntax-table' property
		 ;; then it is quoted text and should *not* cause the
		 ;; user to be asked about a missing attachment.
		 if (let ((props (text-properties-at (match-beginning 0))))
		      (not (or (memq 'syntax-table props)
			       (memq 'face props))))
		 return t
		 finally return nil))
	 ;; ...but doesn't have a part with a filename...
	 (save-excursion
	   (message-goto-body)
	   (not (re-search-forward "^<#part [^>]*filename=" nil t)))
	 ;; ...and that's not okay...
	 (not (y-or-n-p "Attachment mentioned, but no attachment - is that okay?")))
    ;; ...signal an error.
    (error "Missing attachment")))

(add-hook 'message-send-hook 'notmuch-mua-attachment-check)
(add-hook 'message-send-hook 'notmuch-draft--mark-deleted)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; keymappings
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define-key notmuch-show-mode-map "S"
  (lambda ()
    "mark message as spam and advance"
    (interactive)
    (notmuch-show-tag '("+spam" "-inbox" "-ham" "-unread" "-flagged"))
    (notmuch-show-next-open-message t)))

(define-key notmuch-search-mode-map "S"
  (lambda (&optional beg end)
    "mark message as spam and advance"
    (interactive (notmuch-search-interactive-region))
    (notmuch-search-tag (list "+spam" "-inbox" "-ham" "-unread" "-flagged") beg end t)
    (anarcat/notmuch-search-next-thread)))

(define-key notmuch-show-mode-map "H"
  (lambda ()
    "mark message as ham and advance"
    (interactive)
    (notmuch-show-tag '("-spam" "-greyspam" "+inbox" "+ham" "+flagged"))
    (notmuch-show-next-open-message t)))

(define-key notmuch-search-mode-map "H"
  (lambda (&optional beg end)
    "mark message as ham and advance"
    (interactive (notmuch-search-interactive-region))
    (notmuch-search-tag (list "-spam" "-greyspam" "+inbox" "+ham" "+flagged") beg end t)
    (anarcat/notmuch-search-next-thread)))

(define-key notmuch-search-mode-map "u"
  (lambda (&optional beg end)
    "undelete and advance"
    (interactive (notmuch-search-interactive-region))
    (notmuch-search-tag (list "-deleted") beg end t)
    (anarcat/notmuch-search-next-thread)))

(define-key notmuch-search-mode-map "d"
  (lambda (&optional beg end)
    "delete and advance"
    (interactive (notmuch-search-interactive-region))
    (notmuch-search-tag (list "+deleted" "-unread") beg end t)
    (anarcat/notmuch-search-next-thread)))

(define-key notmuch-show-mode-map "d"
  (lambda ()
    "delete current message and advance"
    (interactive)
    (notmuch-show-tag '("+deleted" "-unread"))
    (notmuch-show-next-open-message t)))

;; https://notmuchmail.org/emacstips/#index17h2
(define-key notmuch-show-mode-map "b"
  (lambda (&optional address)
    "Bounce the current message."
    (interactive "sBounce To: ")
    (notmuch-show-view-raw-message)
    (message-resend address)
    (kill-buffer)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; my custom notmuch functions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun anarcat/notmuch-search-next-thread ()
  "Skip to next message from region or point

This is necessary because notmuch-search-next-thread just starts
from point, whereas it seems to me more logical to start from the
end of the region."
  ;; move line before the end of region if there is one
  (unless (= beg end)
    (goto-char (- end 1)))
  (notmuch-search-next-thread))

;; Linking to notmuch messages from org-mode
;; https://notmuchmail.org/emacstips/#index23h2
(require 'org-notmuch nil t)
(autoload 'gnus-alias-determine-identity "gnus-alias" "" t)
(add-hook 'message-setup-hook 'gnus-alias-determine-identity)

(defun anarcat/notmuch-html-convert ()
  """create an HTML part from a Markdown body

This will not work if there are *any* attachments of any form, those should be added after."""
  (interactive)
  (save-excursion
    ;; fetch subject, it will be the HTML version title
    (message "building HTML attachment...")
    (message-goto-subject)
    (beginning-of-line)
    (search-forward ":")
    (forward-char)
    (let ((beg (point))) (end-of-line)
         (setq subject (buffer-substring beg (point))))
    (message "determined title is %s..." subject)
    ;; wrap signature in a <pre>
    (message-goto-signature)
    (forward-line -1)
    ;; save and delete signature which requires special formatting
    (setq signature (buffer-substring (point) (point-max)))
    (delete-region (point) (point-max))
    ;; set region to top of body then end of buffer
    (end-of-buffer)
    (message-goto-body)
    (narrow-to-region (point) (mark))
    ;; run markdown on region
    (setq output-buffer-name "*notmuch-markdown-output*")
    (message "running markdown...")
    (markdown output-buffer-name)
    (widen)
    (save-excursion
      (set-buffer output-buffer-name)
      (end-of-buffer)
      ;; add signature formatted as <pre>
      (insert "\n<pre>")
      (insert signature)
      (insert "</pre>\n")
      (markdown-add-xhtml-header-and-footer subject))
    (message "done the dirty work, re-inserting everything...")
    ;; restore signature
    (message-goto-signature)
    (insert signature)
    (message-goto-body)
    (insert "<#multipart type=alternative>\n")
    (end-of-buffer)
    (insert "<#part type=text/html>\n")
    (insert-buffer output-buffer-name)
    (end-of-buffer)
    (insert "<#/multipart>\n")
    (let ((f (buffer-size (get-buffer output-buffer-name))))
      (message "appended HTML part (%s bytes)" f))))

(message "anarcat's custom notmuch config loaded")
